@@ -33,7 +33,11 @@ const animate = (a, b, duration, ease, render) => new Promise(resolve => {
 const uncollapse = range => {
     if (!range?.collapsed) return range
     const { endOffset, endContainer } = range
-    if (endContainer.nodeType === 1) return endContainer
+    if (endContainer.nodeType === 1) {
+        const node = endContainer.childNodes[endOffset]
+        if (node?.nodeType === 1) return node
+        return endContainer
+    }
     if (endOffset + 1 < endContainer.length) range.setEnd(endContainer, endOffset + 1)
     else if (endOffset > 1) range.setStart(endContainer, endOffset - 1)
     else return endContainer.parentNode
@@ -64,6 +68,21 @@ const { SHOW_ELEMENT, SHOW_TEXT, SHOW_CDATA_SECTION,
 
 const filter = SHOW_ELEMENT | SHOW_TEXT | SHOW_CDATA_SECTION
 
+// needed cause there seems to be a bug in `getBoundingClientRect()` in Firefox
+// where it fails to include rects that have zero width and non-zero height
+// (CSSOM spec says "rectangles [...] of which the height or width is not zero")
+// which makes the visible range include an extra space at column boundaries
+const getBoundingClientRect = target => {
+    let top = Infinity, right = -Infinity, left = Infinity, bottom = -Infinity
+    for (const rect of target.getClientRects()) {
+        left = Math.min(left, rect.left)
+        top = Math.min(top, rect.top)
+        right = Math.max(right, rect.right)
+        bottom = Math.max(bottom, rect.bottom)
+    }
+    return new DOMRect(left, top, right - left, bottom - top)
+}
+
 const getVisibleRange = (doc, start, end, mapRect) => {
     // first get all visible nodes
     const acceptNode = node => {
@@ -104,15 +123,15 @@ const getVisibleRange = (doc, start, end, mapRect) => {
     // find the offset at which visibility changes
     const startOffset = from.nodeType === 1 ? 0
         : bisectNode(doc, from, (a, b) => {
-            const p = mapRect(a.getBoundingClientRect())
-            const q = mapRect(b.getBoundingClientRect())
+            const p = mapRect(getBoundingClientRect(a))
+            const q = mapRect(getBoundingClientRect(b))
             if (p.right < start && q.left > start) return 0
             return q.left > start ? -1 : 1
         })
     const endOffset = to.nodeType === 1 ? 0
         : bisectNode(doc, to, (a, b) => {
-            const p = mapRect(a.getBoundingClientRect())
-            const q = mapRect(b.getBoundingClientRect())
+            const p = mapRect(getBoundingClientRect(a))
+            const q = mapRect(getBoundingClientRect(b))
             if (p.right < end && q.left > end) return 0
             return q.left > end ? -1 : 1
         })
@@ -123,6 +142,29 @@ const getVisibleRange = (doc, start, end, mapRect) => {
     return range
 }
 
+const selectionIsBackward = sel => {
+    const range = document.createRange()
+    range.setStart(sel.anchorNode, sel.anchorOffset)
+    range.setEnd(sel.focusNode, sel.focusOffset)
+    return range.collapsed
+}
+
+const setSelectionTo = (target, collapse) => {
+    let range
+    if (target.startContainer) range = target.cloneRange()
+    else if (target.nodeType) {
+        range = document.createRange()
+        range.selectNode(target)
+    }
+    if (range) {
+        const sel = range.startContainer.ownerDocument.defaultView.getSelection()
+        sel.removeAllRanges()
+        if (collapse === -1) range.collapse(true)
+        else if (collapse === 1) range.collapse()
+        sel.addRange(range)
+    }
+}
+
 const getDirection = doc => {
     const { defaultView } = doc
     const { writingMode, direction } = defaultView.getComputedStyle(doc.body)
@@ -376,6 +418,7 @@ export class Paginator extends HTMLElement {
     ]
     #root = this.attachShadow({ mode: 'closed' })
     #observer = new ResizeObserver(() => this.render())
+    #top
     #background
     #container
     #header
@@ -395,65 +438,78 @@ export class Paginator extends HTMLElement {
     #scrollBounds
     #touchState
     #touchScrolled
+    #lastVisibleRange
     constructor() {
         super()
         this.#root.innerHTML = `<style>
         :host {
-            --_gap: 7%;
-            --_margin: 48px;
-            --_max-inline-size: 720px;
-            --_max-block-size: 1440px;
-            --_max-column-count: 2;
-            --_vertical: 0;
-            --_half-gap: calc(var(--_gap) / 2);
-            --_max-width: calc(
-                var(--_vertical) * var(--_max-block-size)
-                + (1 - var(--_vertical)) * var(--_max-inline-size) * var(--_max-column-count)
-            );
-            --_max-height: calc(
-                var(--_vertical) * var(--_max-inline-size) * var(--_max-column-count)
-                + (1 - var(--_vertical)) * var(--_max-block-size)
-            );
-            display: grid;
-            grid-template-columns:
-                minmax(var(--_half-gap), 1fr)
-                minmax(0, var(--_max-width))
-                minmax(var(--_half-gap), 1fr);
-            grid-template-rows:
-                minmax(var(--_margin), 1fr)
-                minmax(0, var(--_max-height))
-                minmax(var(--_margin), 1fr);
+            display: block;
+            container-type: size;
+        }
+        :host, #top {
             box-sizing: border-box;
             position: relative;
             overflow: hidden;
             width: 100%;
             height: 100%;
         }
+        #top {
+            --_gap: 7%;
+            --_margin: 48px;
+            --_max-inline-size: 720px;
+            --_max-block-size: 1440px;
+            --_max-column-count: 2;
+            --_max-column-count-portrait: 1;
+            --_max-column-count-spread: var(--_max-column-count);
+            --_half-gap: calc(var(--_gap) / 2);
+            --_max-width: calc(var(--_max-inline-size) * var(--_max-column-count-spread));
+            --_max-height: var(--_max-block-size);
+            display: grid;
+            grid-template-columns:
+                minmax(var(--_half-gap), 1fr)
+                var(--_half-gap)
+                minmax(0, calc(var(--_max-width) - var(--_gap)))
+                var(--_half-gap)
+                minmax(var(--_half-gap), 1fr);
+            grid-template-rows:
+                minmax(var(--_margin), 1fr)
+                minmax(0, var(--_max-height))
+                minmax(var(--_margin), 1fr);
+            &.vertical {
+                --_max-column-count-spread: var(--_max-column-count-portrait);
+                --_max-width: var(--_max-block-size);
+                --_max-height: calc(var(--_max-inline-size) * var(--_max-column-count-spread));
+            }
+            @container (orientation: portrait) {
+                & {
+                    --_max-column-count-spread: var(--_max-column-count-portrait);
+                }
+                &.vertical {
+                    --_max-column-count-spread: var(--_max-column-count);
+                }
+            }
+        }
         #background {
-            grid-column-start: 1;
-            grid-column-end: 4;
-            grid-row-start: 1;
-            grid-row-end: 4;
+            grid-column: 1 / -1;
+            grid-row: 1 / -1;
         }
         #container {
-            grid-column-start: 2;
-            grid-row-start: 2;
+            grid-column: 2 / 5;
+            grid-row: 2;
             overflow: hidden;
         }
         :host([flow="scrolled"]) #container {
-            grid-column-start: 1;
-            grid-column-end: 4;
-            grid-row-start: 1;
-            grid-row-end: 4;
+            grid-column: 1 / -1;
+            grid-row: 1 / -1;
             overflow: auto;
         }
         #header {
-            grid-column-start: 2;
-            grid-row-start: 1;
+            grid-column: 3 / 4;
+            grid-row: 1;
         }
         #footer {
-            grid-column-start: 2;
-            grid-row-start: 3;
+            grid-column: 3 / 4;
+            grid-row: 3;
             align-self: end;
         }
         #header, #footer {
@@ -475,18 +531,22 @@ export class Paginator extends HTMLElement {
             opacity: .6;
         }
         </style>
-        <div id="background" part="filter"></div>
-        <div id="header"></div>
-        <div id="container"></div>
-        <div id="footer"></div>
+        <div id="top">
+            <div id="background" part="filter"></div>
+            <div id="header"></div>
+            <div id="container"></div>
+            <div id="footer"></div>
+        </div>
         `
 
+        this.#top = this.#root.getElementById('top')
         this.#background = this.#root.getElementById('background')
         this.#container = this.#root.getElementById('container')
         this.#header = this.#root.getElementById('header')
         this.#footer = this.#root.getElementById('footer')
 
         this.#observer.observe(this.#container)
+        this.#container.addEventListener('scroll', () => this.dispatchEvent(new Event('scroll')))
         this.#container.addEventListener('scroll', debounce(() => {
             if (this.scrolled) {
                 if (this.#justAnchored) this.#justAnchored = false
@@ -504,6 +564,50 @@ export class Paginator extends HTMLElement {
             doc.addEventListener('touchend', this.#onTouchEnd.bind(this))
         })
 
+        this.addEventListener('relocate', ({ detail }) => {
+            if (detail.reason === 'selection') setSelectionTo(this.#anchor, 0)
+            else if (detail.reason === 'navigation') {
+                if (this.#anchor === 1) setSelectionTo(detail.range, 1)
+                else if (typeof this.#anchor === 'number')
+                    setSelectionTo(detail.range, -1)
+                else setSelectionTo(this.#anchor, -1)
+            }
+        })
+        const checkPointerSelection = debounce((range, sel) => {
+            const selRange = sel.getRangeAt(0)
+            const backward = selectionIsBackward(sel)
+            if (backward && selRange.compareBoundaryPoints(Range.START_TO_START, range) < 0)
+                this.prev()
+            else if (!backward && selRange.compareBoundaryPoints(Range.END_TO_END, range) > 0)
+                this.next()
+        }, 700)
+        this.addEventListener('load', ({ detail: { doc } }) => {
+            let isPointerSelecting = false
+            doc.addEventListener('pointerdown', () => isPointerSelecting = true)
+            doc.addEventListener('pointerup', () => isPointerSelecting = false)
+            let isKeyboardSelecting = false
+            doc.addEventListener('keydown', () => isKeyboardSelecting = true)
+            doc.addEventListener('keyup', () => isKeyboardSelecting = false)
+            doc.addEventListener('selectionchange', () => {
+                if (this.scrolled) return
+                const range = this.#lastVisibleRange
+                if (!range) return
+                const sel = doc.getSelection()
+                if (!sel.rangeCount) return
+                if (isPointerSelecting && sel.type === 'Range')
+                    checkPointerSelection(range, sel)
+                else if (isKeyboardSelecting || sel.type === 'Caret') {
+                    const selRange = sel.getRangeAt(0).cloneRange()
+                    const backward = selectionIsBackward(sel)
+                    if (!backward) selRange.collapse()
+                    this.#scrollToAnchor(selRange)
+                }
+            })
+            doc.addEventListener('focusin', e => this.scrolled ? null :
+                // NOTE: `requestAnimationFrame` is needed in WebKit
+                requestAnimationFrame(() => this.#scrollToAnchor(e.target)))
+        })
+
         this.#mediaQueryListener = () => {
             if (!this.#view) return
             this.#background.style.background = getBackground(this.#view.document)
@@ -519,11 +623,11 @@ export class Paginator extends HTMLElement {
             case 'margin':
             case 'max-block-size':
             case 'max-column-count':
-                this.style.setProperty('--_' + name, value)
+                this.#top.style.setProperty('--_' + name, value)
                 break
             case 'max-inline-size':
                 // needs explicit `render()` as it doesn't necessarily resize
-                this.style.setProperty('--_' + name, value)
+                this.#top.style.setProperty('--_' + name, value)
                 this.render()
                 break
         }
@@ -539,7 +643,7 @@ export class Paginator extends HTMLElement {
         }
         this.#view = new View({
             container: this,
-            onExpand: () => this.scrollToAnchor(this.#anchor),
+            onExpand: () => this.#scrollToAnchor(this.#anchor),
         })
         this.#container.append(this.#view.element)
         return this.#view
@@ -547,7 +651,7 @@ export class Paginator extends HTMLElement {
     #beforeRender({ vertical, rtl, background }) {
         this.#vertical = vertical
         this.#rtl = rtl
-        this.style.setProperty('--_vertical', vertical ? 1 : 0)
+        this.#top.classList.toggle('vertical', vertical)
 
         // set background to `doc` background
         // this is needed because the iframe does not fill the whole element
@@ -556,9 +660,9 @@ export class Paginator extends HTMLElement {
         const { width, height } = this.#container.getBoundingClientRect()
         const size = vertical ? height : width
 
-        const style = getComputedStyle(this)
+        const style = getComputedStyle(this.#top)
         const maxInlineSize = parseFloat(style.getPropertyValue('--_max-inline-size'))
-        const maxColumnCount = parseInt(style.getPropertyValue('--_max-column-count'))
+        const maxColumnCount = parseInt(style.getPropertyValue('--_max-column-count-spread'))
         const margin = parseFloat(style.getPropertyValue('--_margin'))
         this.#margin = margin
 
@@ -586,7 +690,7 @@ export class Paginator extends HTMLElement {
         if (flow === 'scrolled') {
             // FIXME: vertical-rl only, not -lr
             this.setAttribute('dir', vertical ? 'rtl' : 'ltr')
-            this.style.padding = '0'
+            this.#top.style.padding = '0'
             const columnWidth = maxInlineSize
 
             this.heads = null
@@ -607,7 +711,6 @@ export class Paginator extends HTMLElement {
         const marginalStyle = {
             gridTemplateColumns: `repeat(${marginalDivisor}, 1fr)`,
             gap: `${gap}px`,
-            padding: vertical ? '0' : `0 ${gap / 2}px`,
             direction: this.bookDir === 'rtl' ? 'rtl' : 'ltr',
         }
         Object.assign(this.#header.style, marginalStyle)
@@ -627,7 +730,7 @@ export class Paginator extends HTMLElement {
             vertical: this.#vertical,
             rtl: this.#rtl,
         }))
-        this.scrollToAnchor(this.#anchor)
+        this.#scrollToAnchor(this.#anchor)
     }
     get scrolled() {
         return this.getAttribute('flow') === 'scrolled'
@@ -756,7 +859,6 @@ export class Paginator extends HTMLElement {
             return this.#scrollTo(offset, reason)
         }
         const offset = this.#getRectMapper()(rect).left
-            + this.#margin / 2
         return this.#scrollToPage(Math.floor(offset / this.size) + (this.#rtl ? -1 : 1), reason)
     }
     async #scrollTo(offset, reason, smooth) {
@@ -787,6 +889,9 @@ export class Paginator extends HTMLElement {
         return this.#scrollTo(offset, reason, smooth)
     }
     async scrollToAnchor(anchor, select) {
+        return this.#scrollToAnchor(anchor, select ? 'selection' : 'navigation')
+    }
+    async #scrollToAnchor(anchor, reason = 'anchor') {
         this.#anchor = anchor
         const rects = uncollapse(anchor)?.getClientRects?.()
         // if anchor is an element or a range
@@ -796,28 +901,19 @@ export class Paginator extends HTMLElement {
             const rect = Array.from(rects)
                 .find(r => r.width > 0 && r.height > 0) || rects[0]
             if (!rect) return
-            await this.#scrollToRect(rect, 'anchor')
-            if (select) this.#selectAnchor()
+            await this.#scrollToRect(rect, reason)
             return
         }
         // if anchor is a fraction
         if (this.scrolled) {
-            await this.#scrollTo(anchor * this.viewSize, 'anchor')
+            await this.#scrollTo(anchor * this.viewSize, reason)
             return
         }
         const { pages } = this
         if (!pages) return
         const textPages = pages - 2
         const newPage = Math.round(anchor * (textPages - 1))
-        await this.#scrollToPage(newPage + 1, 'anchor')
-    }
-    #selectAnchor() {
-        const { defaultView } = this.#view.document
-        if (this.#anchor.startContainer) {
-            const sel = defaultView.getSelection()
-            sel.removeAllRanges()
-            sel.addRange(this.#anchor)
-        }
+        await this.#scrollToPage(newPage + 1, reason)
     }
     #getVisibleRange() {
         if (this.scrolled) return getVisibleRange(this.#view.document,
@@ -828,8 +924,10 @@ export class Paginator extends HTMLElement {
     }
     #afterScroll(reason) {
         const range = this.#getVisibleRange()
+        this.#lastVisibleRange = range
         // don't set new anchor if relocation was to scroll to anchor
-        if (reason !== 'anchor') this.#anchor = range
+        if (reason !== 'selection' && reason !== 'navigation' && reason !== 'anchor')
+            this.#anchor = range
         else this.#justAnchored = true
 
         const index = this.#index
@@ -846,6 +944,7 @@ export class Paginator extends HTMLElement {
     async #display(promise) {
         const { index, src, anchor, onLoad, select } = await promise
         this.#index = index
+        const hasFocus = this.#view?.document?.hasFocus()
         if (src) {
             const view = this.#createView()
             const afterLoad = doc => {
@@ -870,6 +969,7 @@ export class Paginator extends HTMLElement {
         }
         await this.scrollToAnchor((typeof anchor === 'function'
             ? anchor(this.#view.document) : anchor) ?? 0, select)
+        if (hasFocus) this.focusView()
     }
     #canGoToIndex(index) {
         return index >= 0 && index <= this.sections.length - 1
@@ -981,11 +1081,16 @@ export class Paginator extends HTMLElement {
             $style.textContent = style
         } else $style.textContent = styles
 
-        this.#background.style.background = getBackground(this.#view.document)
+        // NOTE: needs `requestAnimationFrame` in Chromium
+        requestAnimationFrame(() =>
+            this.#background.style.background = getBackground(this.#view.document))
 
         // needed because the resize observer doesn't work in Firefox
         this.#view?.document?.fonts?.ready?.then(() => this.#view.expand())
     }
+    focusView() {
+        this.#view.document.defaultView.focus()
+    }
     destroy() {
         this.#observer.unobserve(this)
         this.#view.destroy()
