@@ -11,7 +11,7 @@ import { gettext as _ } from 'gettext'
 import * as utils from './utils.js'
 import * as format from './format.js'
 import { exportAnnotations } from './annotations.js'
-import { formatAuthors, makeBookInfoWindow } from './book-info.js'
+import { formatLanguageMap, formatAuthors, makeBookInfoWindow } from './book-info.js'
 
 import WebKit from 'gi://WebKit'
 import { WebView } from './webview.js'
@@ -151,7 +151,7 @@ const BookList = GObject.registerClass({
         this.readCover = utils.memoize(identifier => {
             const path = pkg.cachepath(`${encodeURIComponent(identifier)}.png`)
             try { return GdkPixbuf.Pixbuf.new_from_file(path) }
-            catch (e) { return null }
+            catch { return null }
         })
     }
     loadMore(n) {
@@ -230,6 +230,7 @@ const BookItem = GObject.registerClass({
         'remove-book': { param_types: [Gio.File.$gtype] },
         'export-book': { param_types: [Gio.File.$gtype] },
         'book-info': { param_types: [Gio.File.$gtype] },
+        'open-external-app': { param_types: [Gio.File.$gtype] },
     },
 }, class extends Gtk.Box {
     #item
@@ -240,11 +241,12 @@ const BookItem = GObject.registerClass({
             'remove': () => this.emit('remove-book', this.#item),
             'export': () => this.emit('export-book', this.#item),
             'info': () => this.emit('book-info', this.#item),
+            'open-external-app': () => this.emit('open-external-app', this.#item),
         }))
     }
     update(item, data, cover) {
         this.#item = item
-        const title = data.metadata?.title ?? ''
+        const title = formatLanguageMap(data.metadata?.title)
         this._title.text = title
         this._image.load(cover?.then ? null : cover, title)
         this._progress.label = format.percent(fraction(data.progress))
@@ -260,6 +262,7 @@ const BookRow = GObject.registerClass({
         'remove-book': { param_types: [Gio.File.$gtype] },
         'export-book': { param_types: [Gio.File.$gtype] },
         'book-info': { param_types: [Gio.File.$gtype] },
+        'open-external-app': { param_types: [Gio.File.$gtype] },
     },
 }, class extends Gtk.Box {
     #item
@@ -270,6 +273,7 @@ const BookRow = GObject.registerClass({
             'remove': () => this.emit('remove-book', this.#item),
             'export': () => this.emit('export-book', this.#item),
             'info': () => this.emit('book-info', this.#item),
+            'open-external-app': () => this.emit('open-external-app', this.#item),
         }))
     }
     update(item, data) {
@@ -334,6 +338,7 @@ GObject.registerClass({
             const cover = books.readCover(metadata.identifier)
             makeBookInfoWindow(this.get_root(), metadata, cover)
         },
+        'open-external-app': (_, file) => this.openWithExternalApp(getBooks().getBook(file)),
     }
     actionGroup = utils.addMethods(this, {
         props: ['view-mode'],
@@ -441,6 +446,40 @@ GObject.registerClass({
             if (response === 'remove') getBooks().delete(file)
         })
     }
+    openWithExternalApp(file) {
+        if (!file) return
+        const path = file.get_path()
+        if (!path) return
+
+        const dialog = new Gtk.AppChooserDialog({
+            gfile: file,
+            modal: true,
+            transient_for: this.root,
+        })
+
+        dialog.connect('response', (dialog, response) => {
+            if (response === Gtk.ResponseType.OK) {
+                const app_info = dialog.get_app_info()
+                if (app_info) {
+                    try {
+                        app_info.launch([file], null)
+                    } catch (e) {
+                        console.error(
+                            'Failed to open file with selected application:',
+                            e,
+                        )
+                        this.root.error(
+                            _('Failed to Open'),
+                            _('Could not open the file with the selected application'),
+                        )
+                    }
+                }
+            }
+            dialog.destroy()
+        })
+
+        dialog.show()
+    }
 })
 
 GObject.registerClass({
